
/*************************************************
****      VEGA - Bond managment routines      ****
**** Copyright 1996-2003, Alessandro Pedretti ****
*************************************************/


#include <string.h>
#include <malloc.h>
#include <math.h>

#include "globdef.h"
#include "globvar.h"
#include "globstr.h"

#include "bond.h"


/**** Assign the bond types ****/

VG_BOOL BndAssignTypes(ATOMO *InizAtm, VG_ULONG TotAtm, VG_BOOL ActiveOnly)
{
  ATOMO                 *Atm1, *Atm2;
  char			FieldName[VG_BOND_FIELDLEN];
  VG_UBYTE              Bond, i, j, k, l;
  float                 T;

  VG_BOOL               Ret    = TRUE;

#ifdef __VG_OPENGL
  VG_DLONG              *Store = NULL;

  if (GLOBSW_OPENGL)
    Store = FFPush(InizAtm, TotAtm);
#endif

  /**** Assign BOND atom types ****/

  PrintProg(MSG_BOND_ASSIGNPROG);
  GetPrgPath(FieldName, TRUE);
  strcat(FieldName, VG_BOND_TEMPLATE);
  if ((Ret = AssignFF(FieldName, InizAtm, TotAtm, ActiveOnly, TRUE))) {
    for(Atm1 = InizAtm; Atm1; Atm1 = Atm1 -> Ptr) {
      for(k = 0; k < Atm1 -> NSost; ++k) {
        Atm2 = Atm1 -> Conn[k];
        if (Atm2 -> Num > Atm1 -> Num) {
          for(j = 0; (j < Atm2 -> NSost) && (Atm2 -> Conn[j] != Atm1); ++j);
          Bond = VG_BOND_SINGLE;
          if (j < Atm2 -> NSost) {
            if ((*Atm1 -> Pot.C != '?') && (*Atm2 -> Pot.C != '?')) {
              if (Atm1 -> Pot.DL == Atm2 -> Pot.DL) {
                Bond = *Atm1 -> Pot.C - 0x30;

                /**** Check if the atoms aren't planar ****/

                if ((*Atm1 -> Pot.C == '4') && (*Atm2 -> Pot.C == '4') &&
                    (Atm1 -> NSost == 3) && (Atm2 -> NSost == 3)) {
                  for(i = 0; (i < Atm1 -> NSost) && (Atm1 -> Conn[i] == Atm2); ++i);
                  for(l = 0; (l < Atm2 -> NSost) && (Atm2 -> Conn[l] == Atm1); ++l);

                  T = (float)NormAngle(Torsion(Atm1 -> Conn[i], Atm1, Atm2, Atm2 -> Conn[l]));
                  if ((T > 20.0f) && (T < 160.0f)) Bond = VG_BOND_SINGLE;
                  else if ((T > 200.0f) && (T < 340.0f)) Bond = VG_BOND_SINGLE;
                }
              }
            }
            Atm2 -> Order[j] = Bond;
          }
          Atm1 -> Order[k] = Bond;
        }
      } /* End of for */
    } /* End of for */
  }

#ifdef __VG_OPENGL
  if (Store) {
    FFPop(InizAtm, Store);
    FREE(Store);
  }
#endif
  return Ret;
}


/**** Check if two atoms are already connected ****/

VG_BOOL BndCheck(ATOMO *Atm1, ATOMO *Atm2, VG_UBYTE *Type)
{
  VG_BYTE       k;

  if ((!Atm1 -> NSost) || (!Atm2 -> NSost)) return FALSE;

  for(k = 0; k < Atm2 -> NSost; ++k)
    if (Atm2 -> Conn[k] == Atm1) {
      if (Type) *Type = Atm2 -> Order[k];
      return TRUE;
    }

  return FALSE;
}


/**** Make bond ****/

VG_BOOL BndMake(ATOMO *Atm1, ATOMO *Atm2, VG_BYTE Type)
{
  if ((Atm1 -> NSost >= MAXBOND) || (Atm2 -> NSost >= MAXBOND))
    return FALSE;

  Atm1 -> Conn[Atm1 -> NSost]  = Atm2;
  Atm1 -> Order[Atm1 -> NSost] = Type;
  ++Atm1 -> NSost;
  Atm2 -> Conn[Atm2 -> NSost]  = Atm1;
  Atm2 -> Order[Atm2 -> NSost] = Type;
  ++Atm2 -> NSost;

  return TRUE;
}


#ifdef __VG_OPENGL

/**** Change the bond type ****/

void BndChange(ATOMO *Atm1, ATOMO *Atm2, VG_UBYTE Type)
{
  BndChangeSingle(Atm1, Atm2, Type);
  BndChangeSingle(Atm2, Atm1, Type);
}


/**** Change the bond type to all atoms ****/

void BndChangeAll(ATOMO *Atm, VG_UBYTE Type, VG_BOOL ActiveOnly)
{
  VG_BYTE       k;

  while(Atm) {
    if ((!ActiveOnly) || (Atm -> Active)) {
      for(k = 0; (k < Atm -> NSost); ++k) {
        if ((!ActiveOnly) || (Atm -> Conn[k] -> Active))
          Atm -> Order[k] = Type;
      } /* End of for */
    }
    Atm = Atm -> Ptr;
  } /* End of while */
}


/**** Change the bond type to a single atom ****/

void BndChangeSingle(ATOMO *Atm1, ATOMO *Atm2, VG_UBYTE Type)
{
  VG_BYTE       k;

  for(k = 0; (k < Atm1 -> NSost); ++k)
    if (Atm1 -> Conn[k] == Atm2) {
      Atm1 -> Order[k] = Type;
      break;
    }
}


/**** Delete bond ****/

void BndDelete(ATOMO *Atm1, ATOMO *Atm2)
{
  BndUnlink(Atm1, Atm2);
  BndUnlink(Atm2, Atm1);
}


/**** Delete all bonds ****/

void BndDeleteAll(ATOMO *Atm, VG_BOOL ActiveOnly)
{
  for(; Atm; Atm = Atm -> Ptr) {
    if ((ActiveOnly) && (!Atm -> Active)) continue;
    Atm -> NSost = 0;
  } /* End of while */
}


/**** Find recursively all atoms connected to one ****/

void BndFindConnAtms(VG_BYTE *ConnMtx, ATOMO *Atm, VG_ULONG *Count)
{
  VG_BYTE       k;

  if (ConnMtx[Atm -> Num - 1]) return;

  ConnMtx[Atm -> Num - 1] = TRUE;
  ++*Count;

  for(k = 0; k < Atm -> NSost; ++k)
    BndFindConnAtms(ConnMtx, Atm -> Conn[k], Count);
}


/**** Get the bond length ****/

float BndGetLength(VG_UWORD Elem1, VG_UWORD Elem2)
{
  if (Elem1 == Elem2)
    return (2.0 * GetAtmCovRad(Elem1));

  return (GetAtmCovRad(Elem1) + GetAtmCovRad(Elem2));
}


/**** Get the rotation matrix ****/

VG_BOOL BndGetRotMat(float mat[9], float angle, float x, float y, float z)
{
  float         xx, yy, zz, xy, yz, zx, xs, ys, zs, one_c;

  float         s   = (float)sin(angle * DEG_TO_RAD);
  float         c   = (float)cos(angle * DEG_TO_RAD);
  float         mag = (float)sqrt(x * x + y * y + z * z);


  /**** The matrix can be constructed ? ****/

  if (mag < VG_BOND_ROTPRECISION) return FALSE;

  mag    = 1.0f / mag;

  x     *= mag;
  y     *= mag;
  z     *= mag;

  one_c  = 1.0f - c;

  xx     = x * x * one_c;
  yy     = y * y * one_c;
  zz     = z * z * one_c;
  xy     = x * y * one_c;
  yz     = y * z * one_c;
  zx     = z * x * one_c;

  xs     = x * s;
  ys     = y * s;
  zs     = z * s;

  mat[0] = xx + c;
  mat[1] = xy - zs;
  mat[2] = zx + ys;

  mat[3] = xy + zs;
  mat[4] = yy + c;
  mat[5] = yz - xs;

  mat[6] = zx - ys;
  mat[7] = yz + xs;
  mat[8] = zz + c;

  return TRUE;
}


/**** Rotate a bond ****/

void BndRotate(VG_UBYTE *ConMtx, ATOMO *Atm1, ATOMO *Atm2, float dtor)
{
  ATOMO         *Atm;
  float         m[9], r[3];
  int           index;

  /**** Rotate about the vector r ****/

  r[0] = Atm2 -> x - Atm1 -> x;
  r[1] = Atm2 -> y - Atm1 -> y;
  r[2] = Atm2 -> z - Atm1 -> z;

  if (BndGetRotMat(m, dtor, r[0], r[1], r[2]) == FALSE) return;

  for(Atm = BegAtm; Atm; Atm = Atm -> Ptr) {
    if (ConMtx[Atm -> Num - 1]) {
      r[0] = Atm -> x - Atm1 -> x;
      r[1] = Atm -> y - Atm1 -> y;
      r[2] = Atm -> z - Atm1 -> z;
      Atm -> x = Atm1 -> x + m[0] * r[0] + m[1] * r[1] + m[2] * r[2];
      Atm -> y = Atm1 -> y + m[3] * r[0] + m[4] * r[1] + m[5] * r[2];
      Atm -> z = Atm1 -> z + m[6] * r[0] + m[7] * r[1] + m[8] * r[2];
    }
  } /* End of for */
}


/**** Unlink two atoms ****/

void BndUnlink(ATOMO *Atm1, ATOMO *Atm2)
{
  VG_BYTE       k;

  if (!Atm1 -> NSost) return;

  /**** Find the connected atom ****/

  for(k = 0; (k < Atm1 -> NSost) && (Atm1 -> Conn[k] != Atm2); ++k);

  --Atm1 -> NSost;
  while(k < Atm1 -> NSost) {
    Atm1 -> Conn[k]  = Atm1 -> Conn[k + 1];
    Atm1 -> Order[k] = Atm1 -> Order[k + 1];
    ++k;
  } /* End of while */
}


/**** Unlink all connected atoms to one ****/

void BndUnlinkAll(ATOMO *Atm1)
{
  VG_BYTE       k;

  for(k = 0; (k < Atm1 -> NSost); ++k)
    BndUnlink(Atm1 -> Conn[k], Atm1);
}
#endif


