
/*************************************************
****       VEGA - Subroutines for OpenGL      ****
**** Copyright 1996-2003, Alessandro Pedretti ****
*************************************************/


#include <stdio.h>
#include <string.h>
#include <ctype.h>

#ifdef __WIN32__
#  include <windows.h>
#endif

#include <GL\gl.h>

#include "globdef.h"
#include "globvar.h"
#include "globstr.h"
#include "bond.h"
#include "formats.h"

#include "gl_global.h"
#include "gl_sub.h"


/**** Invert Z coordinates ****/

void InvertZCoord(register ATOMO *Atm)
{
  CloseSrf();

  while(Atm) {
    if (Atm -> Active) Atm -> z *= -1;
    Atm = Atm -> Ptr;
  } /* End of while */
}


/**** Remove in/visible atoms ****/

VG_ULONG RemoveVisAtm(VG_UBYTE Active)
{
  ATOMO                 *Atm, *PrecAtm;
  VG_UBYTE              i;

  VG_ULONG              Count    = 1;
  VG_ULONG              Removed  = 0;

  Atm = PrecAtm = BegAtm;
  while(Atm) {
    if (Atm -> Active == Active) {
      for(i = 0; i < Atm -> NSost; ++i)
        BndUnlink(Atm -> Conn[i], Atm);
      --TotalAtm;
      RemoveAtm(&Atm, &PrecAtm, &BegAtm);
      ++Removed;
    } else {
      PrecAtm    = Atm;
      Atm -> Num = Count++;
      Atm        = Atm -> Ptr;
    }
  } /* End of while */

  if (!BegAtm) MC_New();

  return Removed;
}


/**** Find the segments ****/

VG_ULONG SegFind(ATOMO *InizAtm)
{
  ATOMO        *Atm;

  VG_ULONG     Segs     = 1;
  ATOMO        *PrecAtm = InizAtm;
  VG_BYTE      ChainID  = InizAtm -> ChainID;

  /**** Use the chain strategy ****/

  for(Atm = InizAtm; Atm; Atm = Atm -> Ptr) {
    if (Atm -> Active) {
      Atm -> Flags &= !VG_ATMF_SEGEND;
      if (Atm -> ChainID != ChainID) {
        PrecAtm -> Flags |= VG_ATMF_SEGEND;
        ChainID = Atm -> ChainID;
        ++Segs;
      }
    }
    PrecAtm = Atm;
  } /* End of for */

  /**** Use the residue number strategy ****/

  if (Segs == 1) {
    PrecAtm = InizAtm;
    for(Atm = InizAtm; Atm; Atm = Atm -> Ptr) {
      if ((Atm -> Active) &&
          (PrecAtm -> ResSeq.L < PrecAtm -> ResSeq.L)) {
        PrecAtm -> Flags |= VG_ATMF_SEGEND;
        ++Segs;
      }
      PrecAtm = Atm;
    } /* End of for */
  }

  return Segs;
}


/**** Fix the segments ****/

void SegFix(ATOMO *InizAtm)
{
  VG_ULONG      k = SegFind(InizAtm);

  if (k > 1)
    CatPrintf(stdout, MSG_SEG_FOUND, k);
  else
    CatPrintf(stdout, MSG_SEG_NOTFOUND);
}


/**** Merge the segments ****/

void SegMerge(register ATOMO *Atm)
{
  while(Atm) {
    if (Atm -> Active) {
      Atm -> Flags &= !VG_ATMF_SEGEND;
    }
   Atm = Atm -> Ptr;
  } /* End of while */
}
